;
; This file contains the vector tables
;       
; Written by Brett Browning
; LICENSE:
;  =========================================================================
;    CMDragons'03 RoboCup F180 Firmware Source Code Release
;  -------------------------------------------------------------------------
;    Copyright (C) 2003 Manuela Veloso, Brett Browning, Mike Bowling,
;                       James Bruce; {mmv, brettb, mhb, jbruce}@cs.cmu.edu
;    School of Computer Science, Carnegie Mellon University
;  -------------------------------------------------------------------------
;    This software is distributed under the GNU General Public License,
;    version 2.  If you do not have a copy of this licence, visit
;    www.gnu.org, or write: Free Software Foundation, 59 Temple Place,
;    Suite 330 Boston, MA 02111-1307 USA.  This program is distributed
;    in the hope that it will be useful, but WITHOUT ANY WARRANTY,
;    including MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
; ------------------------------------------------------------------------- 

;-----------------------------------------------------------
; Types
;-----------------------------------------------------------
		.ref    _bad_trap   ; illegal trap
		.ref   	_c_int0     ; entry point to the code
	
PIVR		.set	0701Eh	; Peripheral Interrupt Vector Register
IFR		.set	00006h	; Interrupt Flag register 

; temp
PFDDR		.set	07096h	; Port F Dadta Direction Register
LED_BIT		.set	0040h	; Led bit to turn on
EVAIFRA		.set	0742Fh	; EVA Flag register
T1PINT_BIT	.set	0080h	; t3PINT bit
T1PINT_IFR	.set	0002h	; IFR bit for T3PINT


;-----------------------------------------------------------
; Variable definitions
;-----------------------------------------------------------
CONTEXT_MEM_PTR_BYTES	.set	010h

	.bss	ST0_TEMP, 1
	.bss	ST1_TEMP, 1
	.bss	ST_ACCL, 1
	.bss	ST_ACCH, 1
	.bss	isr_count, 1

	.global	ST0_TEMP
	.global	ST1_TEMP

;-----------------------------------------------------------
; extern labels
;-----------------------------------------------------------
	.global	_vector
	.global	_pvector

;	.ref _test_isr
	.ref _Servo_ISR

;-----------------------------------------------------------
; vectors for teh general ISR
;-----------------------------------------------------------
	
	.sect	"vectors"
_vector: 
RSVECT		B    _c_int0    ; PM 0  Reset Vector            1
INT1	  	B    GISR_ISR	; PM 2	Int level 1		4
;INT2	  	B    GISR_ISR	; PM 4	Int level 2		5
INT2	  	B    _Servo_ISR	;_test_isr	; PM 4	Int level 2		5
INT3	  	B    GISR_ISR	; PM 6	Int level 3		6
INT4	  	B    GISR_ISR	; PM 8	Int level 4		7
INT5	  	B    GISR_ISR	; PM A	Int level 5		8
INT6	  	B    GISR_ISR	; PM C	Int level 6		9
RESERVED	B    _bad_trap  ; PM E  (Analysis Int)         10
SW_INT8   	B    _bad_trap	; PM 10  	User S/W int	-
SW_INT9   	B    _bad_trap	; PM 12  	User S/W int	-
SW_INT10  	B    _bad_trap	; PM 14  	User S/W int	-
SW_INT11  	B    _bad_trap	; PM 16  	User S/W int	-
SW_INT12  	B    _bad_trap	; PM 18  	User S/W int	-
SW_INT13  	B    _bad_trap	; PM 1A  	User S/W int	-
SW_INT14  	B    _bad_trap	; PM 1C  	User S/W int	-
SW_INT15  	B    _bad_trap	; PM 1E  	User S/W int	-
SW_INT16  	B    _bad_trap	; PM 20  	User S/W int	-
TRAP	  	B    _bad_trap	; PM 22  	Trap vector	-
NMI		B    _bad_trap	; PM 24 	Non maskable int
EMU_TRAP  	B    _bad_trap	; PM 26  	Emulator Trap	2
SW_INT20  	B    _bad_trap	; PM 28  	User S/W int	-
SW_INT21  	B    _bad_trap	; PM 2A  	User S/W int	-
SW_INT22  	B    _bad_trap	; PM 2C  	User S/W int	-
SW_INT23  	B    _bad_trap	; PM 2E  	User S/W int	-


            
;**************************************************************
; Stupid password keys
;
;**************************************************************
	.sect	"keys"
	.word #0h	; 40h - all ones disables stupid secutiry
	.word #0h	; 41h - all ones disables stupid secutiry
	.word #0h	; 42h - all ones disables stupid secutiry
	.word #0h	; 43h - all ones disables stupid secutiry

;**************************************************************
; User vector table that is branched to based on the PIVR value from
; the GISR routines.
;
;**************************************************************

                         
	.sect	".pvecs"
_pvector:	
		B	_bad_trap		; Reserved pvector addr offset 
		B	XINT1_ISR		; pvector addr offset 0x01 - XINT1
		B	_bad_trap		; Reserved pvector addr offset 
		B	_bad_trap		; Reserved pvector addr offset 
		B	ADC_ISR			; pvector addr offset 0x04 - ADC
		B	SPI_ISR			; pvector addr offset 0x05 - SPI RX/TX and/or RX Overrun
		B	SCI_RX_ISR		; pvector addr offset 0x06 - SCI RX interrupt
		B	SCI_TX_ISR		; pvector addr offset 0x07 - SCI TX interrupt
		B	_bad_trap		; Reserved pvector addr offset (0x08) 
		B	_bad_trap		; Reserved pvector addr offset (0x09)
		B	_bad_trap		; Reserved pvector addr offset (0x0a)
		B	_bad_trap		; Reserved pvector addr offset (0x0b)
		B	_bad_trap		; Reserved pvector addr offset (0x0c)
		B	_bad_trap		; Reserved pvector addr offset (0x0d)
		B	_bad_trap		; Reserved pvector addr offset (0x0e)
		B	_bad_trap		; Reserved pvector addr offset (0x0f)
		B	_bad_trap		; Reserved pvector addr offset (0x10)
		B	XINT2_ISR		; pvector addr offset 0x11 - XINT2
		B	_bad_trap		; Reserved pvector addr offset (0x12)
		B	_bad_trap		; Reserved pvector addr offset (0x13)
		B	_bad_trap		; Reserved pvector addr offset (0x14)
		B	_bad_trap		; Reserved pvector addr offset (0x15)
		B	_bad_trap		; Reserved pvector addr offset (0x16)
		B	_bad_trap		; Reserved pvector addr offset (0x17)
		B	_bad_trap		; Reserved pvector addr offset (0x18)
		B	PDPB_ISR		; pvector addr offset 0x19 - PDP interrupt 
		B	_bad_trap		; Reserved pvector addr offset (0x1a)
		B	_bad_trap		; Reserved pvector addr offset (0x1b)
		B	_bad_trap		; Reserved pvector addr offset (0x1c)
		B	_bad_trap		; Reserved pvector addr offset (0x1d)
		B	_bad_trap		; Reserved pvector addr offset (0x1e)
		B	XINT3_ISR		; pvector addr offset 0x1f - XINT3
		B	PDPA_ISR		; pvector addr offset 0x20 - PDP interrupt
		B	CMP1_ISR		; pvector addr offset 0x21 - Full Compare 1  interrupt
		B	CMP2_ISR		; pvector addr offset 0x22 -              2  interrupt
		B	CMP3_ISR		; pvector addr offset 0x23 -              3 interrupt
		B	CMP4_ISR		; pvector addr offset 0x24 - Full Compare 4  interrupt
		B	CMP5_ISR		; pvector addr offset 0x25 -              5  interrupt
		B	CMP6_ISR		; pvector addr offset 0x26 -              6  interrupt
		B	TPINT1_ISR		; pvector addr offset 0x27 - T1 Period interrupt
		B	TCINT1_ISR		; pvector addr offset 0x28 -    Compare interrupt
		B	TUFINT1_ISR		; pvector addr offset 0x29 -    Underflow interrupt
		B	TOFINT1_ISR		; pvector addr offset 0x2a -    Overflow interrupt
		B	TPINT2_ISR		; pvector addr offset 0x2b - T2 Period interrupt
		B	TCINT2_ISR		; pvector addr offset 0x2c -    Compare interrupt
		B	TUFINT2_ISR		; pvector addr offset 0x2d -    Underflow interrupt
		B	TOFINT2_ISR		; pvector addr offset 0x2e -    Overflow interrupt
		B	TPINT3_ISR		; pvector addr offset 0x2f - T3 Period interrupt
		B	TCINT3_ISR		; pvector addr offset 0x30 -    Compare interrupt
		B	TUFINT3_ISR		; pvector addr offset 0x31 -    Underflow interrupt
		B	TOFINT3_ISR		; pvector addr offset 0x32 -    Overflow interrupt
		B	CAP1_ISR		; pvector addr offset 0x33 - Capture 1 interrupt
		B	CAP2_ISR		; pvector addr offset 0x34 - Capture 2 interrupt
		B	CAP3_ISR		; pvector addr offset 0x35 - Capture 3 interrupt
		B	CAP4_ISR		; pvector addr offset 0x36 - Capture 4 interrupt
		B	CAP5_ISR		; pvector addr offset 0x37 - Capture 5 interrupt
		B	CAP6_ISR		; pvector addr offset 0x38 - Capture 6 interrupt
		B	_bad_trap		; Reserved pvector addr offset (0x39)
		B	_bad_trap		; Reserved pvector addr offset (0x3a)
		B	_bad_trap		; Reserved pvector addr offset (0x3b)
		B	_bad_trap		; Reserved pvector addr offset (0x3c)
		B	_bad_trap		; Reserved pvector addr offset (0x3d)
		B	_bad_trap		; Reserved pvector addr offset (0x3e)
		B	_bad_trap		; Reserved pvector addr offset (0x3f)
		B	CANMB_ISR		; pvector addr offset 0x40 - CAN Mailbox interrupt
		B	CANER_ISR		; pvector addr offset 0x41 - CAN error interrupt
		
		.text
;***************************************************************
; General Interrupt Service Routines
;***************************************************************
GISR_ISR:
		SST	#0, ST0_TEMP	;Auto page-0 DP	addressing is used to 
		SST	#1, ST1_TEMP	; save status registers to RAM.
		LDP	#ST_ACCL>>7	; change the dp to context stack
		SACL ST_ACCL		; save lower 16-bits of ACC
		SACH ST_ACCH		; save upper 16-bits of ACC
		LDP #PIVR >> 7		; load the PIVR register
		LACC PIVR,1		; load the ACC with the PIVR vector 
					;shifted by 1 to account for double size table
		LDP #0
		ADD #_pvector		; add the pvectors offset
		BACC			; branch to the pvector 
					; really we should just read teh table 
					; and branch using that but this will do for now
		
ISR_DONE:
		LDP	#ST_ACCL >> 7
		LACC ST_ACCH,16			; reload the upper 16 bits of ACC from stack
		OR ST_ACCL			; reload bottom 16 bits of ACC from stack
		LDP	#0			; redo Auto page 0
		LST	#1, ST1_TEMP		; reload  status register
		LST	#0, ST0_TEMP		; reload status register
		CLRC	INTM			; clear the interrupt mask
		RET
		

;***************************************************************
; Interrupt Service Routines
;***************************************************************

TPINT1_ISR:

	LDP	#isr_count >> 7	; check to see if we should run code
	LACC	isr_count,0
	ADD	#1			; increment isr_count and store it
	SACL	isr_count
	AND	#4h			; check to see if it is time to branch
	BCND	isr_cleanup, EQ		; Branch if isr_count != 0

	LACC	#0,0			; reset the isr_count
	SACL	isr_count

	LDP 	#PFDDR >> 7
	LACC	PFDDR
	XOR	#LED_BIT		; toggle the led
	SACL	PFDDR			; set the bit to teh PFDDR
	
isr_cleanup:
	LDP	#EVAIFRA >> 7		; clear the interrupt flag
	LACC	EVAIFRA
	OR	#T1PINT_BIT
	SACL	EVAIFRA

	LDP	#IFR >> 7		; clear the interrupt flag
	LACC	IFR			; now clear IFR bit
	OR	#T1PINT_IFR
	SACL	IFR
	
	B	ISR_DONE		; all done so clean up



;---------------------------------------------------------
; Default ISR's go to bad_trap
;---------------------------------------------------------

XINT1_ISR		
ADC_ISR	
SPI_ISR		
SCI_RX_ISR	
SCI_TX_ISR	
WD_RTI_ISR	
XINT2_ISR		
XINT3_ISR		
PDPA_ISR		
PDPB_ISR		
CMP1_ISR		
CMP2_ISR		
CMP3_ISR		
CMP4_ISR		
CMP5_ISR		
CMP6_ISR		
SCMP1_ISR		
SCMP2_ISR		
SCMP3_ISR		
TOFINT1_ISR	
TOFINT2_ISR	
TOFINT3_ISR	
TCINT1_ISR
TCINT2_ISR	
TCINT3_ISR	
TPINT2_ISR	
TPINT3_ISR	
TUFINT1_ISR	
TUFINT2_ISR	
TUFINT3_ISR	
CAP1_ISR		
CAP2_ISR		
CAP3_ISR		
CAP4_ISR		
CAP5_ISR		
CAP6_ISR		
CANMB_ISR
CANER_ISR
PHANTOM 	LDP	#0			; go to first data page of B2
		B	_bad_trap		; branch to bad trap function

		.end