/*
 * TITLE:  EVx.c
 *
 * DESCRIPTION: This file encapsulates the TMS320LF2407 Event Manager Units of which there
 *    are two. CPU. The software here is based on the TMS320LF2407 documentation 
 *    and App notes -
 *    SPRU357  - DSP Controllers REference Guide : System and Peripherals
 *    
 *
 * WRITTEN BY: Brett Browning
 *    Ravi Balasubarasaman
 *
 * REVISION HISTORY:
 * 10 Feb 2001  - Created (BB)
 *
 * TO DO -
 *
 */
/* LICENSE:
  =========================================================================
    CMDragons'03 RoboCup F180 Firmware Source Code Release
  -------------------------------------------------------------------------
    Copyright (C) 2003 Manuela Veloso, Brett Browning, Mike Bowling,
                       James Bruce; {mmv, brettb, mhb, jbruce}@cs.cmu.edu
    School of Computer Science, Carnegie Mellon University
  -------------------------------------------------------------------------
    This software is distributed under the GNU General Public License,
    version 2.  If you do not have a copy of this licence, visit
    www.gnu.org, or write: Free Software Foundation, 59 Temple Place,
    Suite 330 Boston, MA 02111-1307 USA.  This program is distributed
    in the hope that it will be useful, but WITHOUT ANY WARRANTY,
    including MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  ------------------------------------------------------------------------- */
#include "lf2407.h"
#include "board.h"
#include "adc.h"

/******************************* TYPES ****************************************/


/*
 * InitEvx -
 *
 * This funciton initialises the EVx module
 *
 * RETURN VALUE: TRUE on success, FALSE on failure
 */
int initADC(ADCParams *params)
{
  int  i;
  ushort val = 0;
 
  /* enable the system clock to enable the module */
  SYS_REGS.scsr1 |= SCSR1_ADCEN;
 
  // setup the control
  // cascaded (ie one 16 bit sequence)
  // stop during debugging steps
  // run at full speed
  // continuous mode
  if (params->continuous)
    val |= ADCCTRL_CONT;
  if (params->cascade)
    val |= ADCCTRL_CASCADE;

  ADC_REGS.adcctrl[0] = (ADCCTRL_RUNSTOP | val);

  /* clear the interrupt flags */
  ADC_REGS.adcctrl[1] &= ~(ADC_SEQ1_IEN1 | ADC_SEQ1_IEN2);
  ADC_REGS.adcctrl[1] &= ~(ADC_SEQ2_IEN1 | ADC_SEQ2_IEN2);
}

int ADCStartConversion(ADCSet *adcset)
{
  ushort i, val, reg, shift;
  if (adcset->reset) {
    // should ensure conversions have stopped
    /* reset the sequencers -- NB the NOP in between */
    ADC_REGS.adcctrl[1] |= ADC_SEQ1_RST;
    i++; 
    ADC_REGS.adcctrl[1] |= ADC_SEQ2_RST;
    i++; 
  }

  // set the conversion registers
  ADC_REGS.maxconv = 0;

  for (i = 0; i < adcset->nr_conversions; i++) {
    reg = i / ADC_CHSEL_CHPERREG;
    shift = (i % ADC_CHSEL_CHPERREG) * ADC_CHSEL_SHIFT;
    ADC_REGS.chselseq[reg] &= ~(ADC_CHSEL_MASK << shift);

    val = adcset->channels[i] & ADC_CHSEL_MASK;
    ADC_REGS.chselseq[reg] |= (val << shift);
  }

  // clear out rest
  for (; i < NR_ADC_RESULTS; i++) {
    reg = i / ADC_CHSEL_CHPERREG;
    shift = (i % ADC_CHSEL_CHPERREG) * ADC_CHSEL_SHIFT;
    ADC_REGS.chselseq[reg] &= ~(ADC_CHSEL_MASK << shift);
  }

  ADC_REGS.maxconv = adcset->nr_conversions & MAXCONV_SEQ_MASK;

  // now start conversion
  ADC_REGS.adcctrl[1] |= ADC_SEQ1_SOC | ADC_SEQ2_SOC;

  // wait approx four nops to make sure conversion is in progress
  i++;
  i++;
  i++;
  i++;
}

inline int ADCIsComplete(void)
{
  return ((ADC_REGS.adcctrl[1] & ADC_SEQ1_IFLAG) &&
	  (ADC_REGS.adcctrl[1] & ADC_SEQ2_IFLAG));
}

inline ushort ADCReadResult(int resi)
{
  //  return ((ADC_REGS.result[resi] >> ADC_RESULT_SHIFT) & ADC_RESULT_MASK);
  return ((ADC_REGS.result[resi] >> ADC_RESULT_SHIFT));
}


