/*
 * TITLE:		ADC.h
 *
 * DESCRIPTION:	This file encapsulates the TMS320LF2407 ADC
 * The software here is based on the TMS320LF2407 documentation 
 * and App notes -
 *	SPRU357		- DSP Controllers REference Guide : System and Peripherals
 *
 * WRITTEN BY:	Brett Browning
 */
/* LICENSE:
  =========================================================================
    CMDragons'03 RoboCup F180 Firmware Source Code Release
  -------------------------------------------------------------------------
    Copyright (C) 2003 Manuela Veloso, Brett Browning, Mike Bowling,
                       James Bruce; {mmv, brettb, mhb, jbruce}@cs.cmu.edu
    School of Computer Science, Carnegie Mellon University
  -------------------------------------------------------------------------
    This software is distributed under the GNU General Public License,
    version 2.  If you do not have a copy of this licence, visit
    www.gnu.org, or write: Free Software Foundation, 59 Temple Place,
    Suite 330 Boston, MA 02111-1307 USA.  This program is distributed
    in the hope that it will be useful, but WITHOUT ANY WARRANTY,
    including MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  ------------------------------------------------------------------------- */
#ifndef _ADC_H
#define _ADC_H

#include "lf2407.h"

/******************************* TYPES ****************************************/

// simple fr now
// now interrupts, simple sequencing
typedef struct tagADCParams {
  int reset;  // unused
  int intren; // unused
  int intrmode; // unused
  int cascade;
  int continuous;
} ADCParams;

typedef struct tagADCSet {
  ushort nr_conversions;
  int reset;
  ushort channels[NR_ADC_RESULTS];
} ADCSet;

/******************************* PROTOTYPES ***********************************/

/*
 * InitADC -
 *
 * This funciton initialises the ADC module
 *
 * RETURN VALUE: TRUE on success, FALSE on failure
 */
int initADC(ADCParams *params);

int ADCStartConversion(ADCSet *aset);

// check EOS flag -- do not sue in continuous mode
inline int ADCIsComplete(void);

inline ushort ADCReadResult(int resi);

#endif /* _ADC_ */
