/*
 * TITLE:		EVx.h
 *
 * DESCRIPTION:	This file encapsulates the TMS320LF2407 Event Manager Units of which there
 *				are two. CPU. The software here is based on the TMS320LF2407 documentation 
 *				and App notes -
 *				SPRU357		- DSP Controllers REference Guide : System and Peripherals
 *				
 *
 * WRITTEN BY:	Brett Browning
 *				Ravi Balasubarasaman
 *
 * REVISION HISTORY:
 * 10 Feb 2001		- Created (BB)
 *
 */

/* LICENSE:
  =========================================================================
    CMDragons'03 RoboCup F180 Firmware Source Code Release
  -------------------------------------------------------------------------
    Copyright (C) 2003 Manuela Veloso, Brett Browning, Mike Bowling,
                       James Bruce; {mmv, brettb, mhb, jbruce}@cs.cmu.edu
    School of Computer Science, Carnegie Mellon University
  -------------------------------------------------------------------------
    This software is distributed under the GNU General Public License,
    version 2.  If you do not have a copy of this licence, visit
    www.gnu.org, or write: Free Software Foundation, 59 Temple Place,
    Suite 330 Boston, MA 02111-1307 USA.  This program is distributed
    in the hope that it will be useful, but WITHOUT ANY WARRANTY,
    including MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  ------------------------------------------------------------------------- */
#ifndef _EVX_H
#define _EVX_H

/******************************* TYPES ****************************************/

/* channel bit fields */
#define EVX_CH1			0x0001
#define EVX_CH2			0x0002
#define EVX_CH3			0x0004
#define EVX_CH4			0x0008
#define EVX_CH5			0x0010
#define EVX_CH6			0x0020
#define EVX_CH7			0x0040
#define EVX_CH8			0x0080
#define EVX_CH9			0x0100
#define EVX_CH10		0x0200
#define EVX_CH11		0x0400
#define EVX_CH12		0x0800

#define EVA_CHANNELS	(EVX_CH1 | EVX_CH2 | EVX_CH3 | EVX_CH4 | EVX_CH5 | EVX_CH6)
#define EVB_CHANNELS	(EVX_CH7 | EVX_CH8 | EVX_CH9 | EVX_CH10 | EVX_CH11 | EVX_CH12)

#define EVX_T1PWM		0x1000
#define EVX_T2PWM		0x2000
#define EVX_T3PWM		0x4000
#define EVX_T4PWM		0x8000

/* QEP pins */
#define EVX_QEP1_2		0x0001
#define EVX_QEP3_4		0x0002

/* capture pins */
#define EVX_CAP1		0x0001
#define EVX_CAP2		0x0002
#define EVX_CAP3		0x0004
#define EVX_CAP4		0x0008
#define EVX_CAP5		0x0010
#define EVX_CAP6		0x0020


/******************************* PROTOTYPES ***********************************/

/*
 * InitEvx -
 *
 * This funciton initialises the EVx module
 *
 * RETURN VALUE: TRUE on success, FALSE on failure
 */
int InitEvx(int a);

/*
 * SetGPTimer -
 *
 * This function sets the GP timer of the appropriate EVx module
 * to the settings specified in the parameter list.
 *
 * RETURN VALUE: TRUEon success, FALSE on failure
 */
int SetGPTimer(int ch, int mode);

/*
 * InitPWM -
 *
 * This function initialises the channels to run as standard PWM
 * outputs. 
 *
 * RETURN VALUE: TRUE on success, FALSE on failure
 */
int InitPWM(int chs, ushort period, ushort val);

/*
 * SetPWM -
 *
 * This funciton sets the PWM value of the given channel
 */
void SetPWM(int ch, ushort duty);

/*
 * InitDBPWM -
 *
 * This function initialises the deadband operation of the PWM channel.
 * Bit fields indicate which channels are to be used. Channels must be paired
 *
 * RETURN VALUE: RUEon success, FALSE on failure
 */
int InitDBPWM(ushort chs, ushort pol, ushort per, ulong dband);

/*
 * SetDBPWM -
 *
 * This function sets the dead band duty cycle 
 */
void SetDBPWM(ushort ch, ushort duty);

/*
 * getDBPWM -
 *
 * This function returns the dead band duty cycle 
 */
inline ushort getDBPWM(ushort ch);

/*
 * InitQDEC -
 *
 * This function initialises the QDEC channels on the appropriate EV module.
 *
 * RETURN VALUE: TRUE on success, FALSE on failure
 */
int InitQDEC(ushort ch);

/*
 * ReadQDEC -
 *
 * This function reads the QDEC value from the appropriate count register
 *
 * RETURN VALUE: The QDEC count as a 16bit entity.
 */
ushort ReadQDEC(ushort ch);

#endif /* _EVX_ */
