/*
 * TITLE:		SCI.h
 *
 * DESCRIPTION:	This file encapsulates the SCI module on the TMS320LF2407 DSP. 
 *				The code is based on the TMS320LF2407 documentation and App notes as -
 *				SPRU357		- DSP Controllers REference Guide : System and Peripherals
 *
 * WRITTEN BY:	Brett Browning
 *				Ravi Balasubarasaman
 *
 * REVISION HISTORY:
 * 10 Feb 2001		- Created (BB)
 *
 */

/* LICENSE:
  =========================================================================
    CMDragons'03 RoboCup F180 Firmware Source Code Release
  -------------------------------------------------------------------------
    Copyright (C) 2003 Manuela Veloso, Brett Browning, Mike Bowling,
                       James Bruce; {mmv, brettb, mhb, jbruce}@cs.cmu.edu
    School of Computer Science, Carnegie Mellon University
  -------------------------------------------------------------------------
    This software is distributed under the GNU General Public License,
    version 2.  If you do not have a copy of this licence, visit
    www.gnu.org, or write: Free Software Foundation, 59 Temple Place,
    Suite 330 Boston, MA 02111-1307 USA.  This program is distributed
    in the hope that it will be useful, but WITHOUT ANY WARRANTY,
    including MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  ------------------------------------------------------------------------- */
#ifndef _SCI_H
#define _SCI_H

#include "lf2407.h"

/**************************** TYPES ********************************/

#define SCI_NO_PARITY			0
#define SCI_EVEN_PARITY			1
#define SCI_ODD_PARITY			2

#define SCI_TXEN				0x01
#define SCI_RXEN				0x02
#define SCI_TX_IEN				0x04
#define SCI_RX_IEN				0x08

#define SCI_TX_HIPRIORITY		0x10
#define SCI_RX_HIPRIORITY		0x10

typedef struct {
	ushort	nr_chars;
	ushort	parity;
	ushort	nr_stop;
	ushort	baud;
	ushort	ctrl;
/*	funcp	scivec; */
} SCIParam;


/**************************** TYPES ********************************/
/*
 * InitSCI -
 *
 * This funciton initialises the SCI port on the LF2407 to the appropriate
 * settings 
 *
 * RETURN VALUE: TRUE on success, FALSE on faiolure
 */
int InitSCI(SCIParam *sciparam);

/*
 * SCISend
 *
 * This function sends a single byte through the serial port 
 *
 * RETURN VALUE: TRUE on success, FALSE on failure
 */
int SCISend(ushort data);

/*
 * SCISendPacket -
 *
 * This function sends a packet out the serial port
 */
void SCISendPacket(ushort *data, ushort len);

/*
 * SCISendString -
 *
 * This function sends a string out the serial port
 */
void SCISendString(char *data);

/*
 * SCISendInt -
 *
 * This function sends an int value in hex format to the serial
 * port, which must be initialised
 * Hexchar is a bool that defines if it should send th e'0x' characthers at teh start
 */
void SCISendInt(ushort d, ushort hexchar);

/*
 * SCIRecv -
 *
 * This function polls the serial port until new data has arrived
 */
ushort SCIRecv(void);

/*
 * SCIStatus -
 *
 * This funciton returns a bit field indicating teh status of the serial
 * device.
 *
 * rETURN VALUE: a bit field indicating the status
 */
ushort SCIStatus(void);


#endif /* _SCI_H */
